#include <stdio.h>
#include <stdint.h>
#include <sys/io.h>
#include <unistd.h>
#include "monkey.h"

/* The frequency freq is to be played, which is given in Hz.
The duration is given in RTC ticks through the variable time.
(RTC oscillates with frequency 18.2 Hz.) */
void tm_sound(float freq, uint32_t time) {
	unsigned cnt;
	/* The PIT (programmable interval timer) has 3 channels (0..2) operating in different modes.
	Channel 2 output is connected to the PC speaker.
	Tell PIT that next, the channel 2 will be programmed.
	The frequency will be loaded in two bytes: f
	rst the low byte, and then the high byte.
	The generated output should be a square wave (rather than something else).
	Moreover, the frequency will be provided in binary (rather than in BCD).
	Details: http://wiki.osdev.org/Programmable_Interval_Timer */
	outb_p(0xb6, 0x43);
	if(freq>0) {
		/* The PIT oscillates at 1.193181818181... MHz = 1193181.818181... Hz.
		The PC speaker’s position should be changed each 1/freq seconds,
		which is 1193181.818181.../freq units of the PIT oscillator. */
		cnt = 1193181.81818181818181 / freq;
		// Provide the low byte of the frequency to PIT.
		outb_p(cnt & 0x00ff, 0x42);
		// Provide the high byte of the frequency to PIT.
		outb_p((cnt & 0xff00) >> 8, 0x42);
		// Turn the speaker on and let it obtain the signal from the channel 2 of PIT.
		outb(inb_p(0x61) | 3, 0x61);
		/* Some simple ways to incur a delay:
		An RTC tick lasts 1/18.2 s, so time RTC−ticks last time/18.2 s =
		time∗10^6/18.2 microseconds = 54945.054945055∗time microseconds. */
		usleep(time/**54945.054945055*/); // let it be turned on time∗54945.054945055 microseconds.
		// Turn the speaker off and disconnect it from the channel 2 of PIT.
		outb_p(inb_p(0x61) & 0xfc, 0x61);
	}
	else usleep(time/**54945.054945055*/); // wait time∗54945.054945055 microseconds.
}

//−1 means pause, 0 means stop.
float music[] = {
	440 , 329.6, 440 , 329.6, 440 , 329.6, 440,
	523.4, 440 , 523.4, 440 , 523.4, 440 , 523.4,
	659.2, 523.4, 659.2, 587.4, 493.9, 587.4,
	523.4, 523.4, 440 , 523.4, 493.9, 493.9, 659.2,
	440, 329.6, 440 , 329.6, 440 , 329.6, 440,
	523.4, 440 , 523.4, 440 , 523.4, 440 , 523.4,
	659.2, 523.4, 659.2, 587.4, 493.9, 587.4,
	523.4, 440 , 493.9, 415 , 440 ,-1 ,
	-1 , 329.6, 349.2, 392 , 349.2, 329.6, 293.7, 261.7,
	-1 , 329.6, 349.2, 392 , 349.2, 329.6, 293.7, 261.7,
	-1 , 329.6, 349.2, 392 , 440 , 392 , 349.2, 329.6,
	293.7, 293.7, 349.2, 329.6, 293.7,-1 ,
	-1 , 293.7, 329.6, 349.2, 392 , 349.2, 329.6, 293.7,
	261.7, 329.6, 293.7, 261.7,-1 , 329.6,
	329.6, 311.1, 311.1, 261.7, 261.7, 246.95, 246.95, 220,
	220,
	0
};

uint16_t music_del[] = {
	10 , 5 , 5 , 5 , 5 , 5 , 5,
	10 , 5 , 5 , 5 , 5 , 5 , 5,
	10 , 5 , 5 , 10 , 5 , 5 ,
	5 , 5 , 5 , 5 , 5 , 5 , 10,
	10 , 5 , 5 , 5 , 5 , 5 , 5,
	10 , 5 , 5 , 5 , 5 , 5 , 5,
	10 , 5 , 5 , 10 , 5 , 5 ,
	5 , 5 , 5 , 5 , 10 , 10 ,
	5 , 5 , 5 , 5 , 5 , 5 , 5 , 5 ,
	5 , 5 , 5 , 5 , 5 , 5 , 5 , 5 ,
	5 , 5 , 5 , 5 , 5 , 5 , 5 , 5 ,
	5 , 5 , 5 , 5 , 5 , 10 ,
	5 , 5 , 5 , 5 , 5 , 5 , 5 , 5 ,
	10 , 5 , 5 , 10 , 5 , 5 ,
	5 , 5 , 5 , 5 , 5 , 5 , 5 , 5 ,
	40
};
int main(void) {
	int i;
	ioperm(0x42,2,1);
	ioperm(0x61,1,1);
	ioperm(0x80,1,1);
	for(i=0; monkey_notes[i]; i++)
	tm_sound(monkey_notes[i], monkey_times[i]);
	return 0;
}

